@@ -33,9 +33,11 @@ Authors:
 #-----------------------------------------------------------------------------
 
 # stdlib imports
+import logging
 import pkg_resources
 import os
 import signal
+import tempfile
 
 # System library imports
 from PySide import QtCore, QtGui
@@ -165,6 +167,30 @@ class NXConsoleApp(BaseIPythonApplication, IPythonConsoleApp):
         super(NXConsoleApp, self).parse_command_line(argv)
         self.build_kernel_argv(argv)
 
+    def init_dir(self):
+        """Initialize NeXpy home directory"""
+        home_dir = os.path.realpath(os.path.expanduser('~'))
+        nexpy_dir = os.path.join(home_dir, '.nexpy')
+        if not os.path.exists(nexpy_dir):
+            parent = os.path.dirname(nexpy_dir)
+            if not os.access(parent, os.W_OK):
+                nexpy_dir = tempfile.mkdtemp()
+            else:
+                os.mkdir(nexpy_dir)
+        for subdirectory in ['functions', 'plugins', 'readers', 'scripts']:
+            directory = os.path.join(nexpy_dir, subdirectory)
+            if not os.path.exists(directory):
+                os.mkdir(directory)
+        global _nexpy_dir
+        _nexpy_dir = nexpy_dir
+
+    def init_log(self):
+        log_file = os.path.join(_nexpy_dir, 'nexpy.log')
+        logging.basicConfig(filename=log_file, filemode='a',
+            format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
+            level=logging.INFO)
+        logging.info('NeXpy launched')
+
     def init_tree(self):
         """Initialize the NeXus tree used in the tree view."""
         global _tree
@@ -190,7 +216,8 @@ class NXConsoleApp(BaseIPythonApplication, IPythonConsoleApp):
         s = ("import nexpy\n"
              "import nexpy.api.nexus as nx\n"
              "from nexpy.api.nexus import NXgroup, NXfield, NXattr, NXlink\n"
-             "from nexpy.api.nexus import NXFile")
+             "from nexpy.api.nexus import NXFile\n"
+             "from nexpy.gui.plotview import NXPlotView")
         exec s in self.window.user_ns
         
         s = ""
@@ -204,7 +231,9 @@ class NXConsoleApp(BaseIPythonApplication, IPythonConsoleApp):
             s = ''.join(f.readlines())
             exec s in self.window.user_ns
         except:
-            s = ("import numpy as np\n"
+            s = ("import sys\n"
+                 "import os\n"
+                 "import numpy as np\n"
                  "import numpy.ma as ma\n"
                  "import scipy as sp\n"
                  "import matplotlib as mpl\n"
@@ -236,6 +265,8 @@ class NXConsoleApp(BaseIPythonApplication, IPythonConsoleApp):
     @catch_config_error
     def initialize(self, argv=None):
         super(NXConsoleApp, self).initialize(argv)
+        self.init_dir()
+        self.init_log()
         self.init_tree()
         self.init_gui()
         self.init_shell()
