@@ -10,6 +10,7 @@
 #-----------------------------------------------------------------------------
 
 import imp
+import logging
 import os
 import re
 import sys
@@ -38,13 +39,209 @@ def wrap(text, length):
         if w is words[-1]: lines.append(line)
     return '\n'.join(lines)
 
+def natural_sort(key):
+    import re
+    return [int(t) if t.isdigit() else t for t in re.split(r'(\d+)', key)]    
 
-class PlotDialog(QtGui.QDialog):
+
+class BaseDialog(QtGui.QDialog):
+    """Base dialog class for NeXpy dialogs"""
+ 
+    def __init__(self, parent=None):
+
+
+        self.accepted = False
+        from nexpy.gui.consoleapp import _mainwindow
+        self.mainwindow = _mainwindow
+        self.treeview = self.mainwindow.treeview
+        self.default_directory = _mainwindow.default_directory
+        self.import_file = None     # must define in subclass
+        self.nexus_filter = ';;'.join((
+             "NeXus Files (*.nxs *.nx5 *.h5 *.hdf *.hdf5)",
+	         "Any Files (*.* *)"))
+
+        if parent is None:
+            parent = self.mainwindow
+        super(BaseDialog, self).__init__(parent)
+
+    def buttonbox(self, save=False):
+        """
+        Creates a box containing the standard Cancel and OK buttons.
+        """
+        buttonbox = QtGui.QDialogButtonBox(self)
+        buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        if save:
+            buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|
+                                         QtGui.QDialogButtonBox.Save)
+        else:
+            buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|
+                                         QtGui.QDialogButtonBox.Ok)
+        buttonbox.accepted.connect(self.accept)
+        buttonbox.rejected.connect(self.reject)
+        return buttonbox
+
+    def filebox(self):
+        """
+        Creates a text box and button for selecting a file.
+        """
+        self.filebutton =  QtGui.QPushButton("Choose File")
+        self.filebutton.clicked.connect(self.choose_file)
+        self.filename = QtGui.QLineEdit(self)
+        self.filename.setMinimumWidth(300)
+        filebox = QtGui.QHBoxLayout()
+        filebox.addWidget(self.filebutton)
+        filebox.addWidget(self.filename)
+        return filebox
+ 
+    def directorybox(self):
+        """
+        Creates a text box and button for selecting a directory.
+        """
+        self.directorybutton =  QtGui.QPushButton("Choose Directory")
+        self.directorybutton.clicked.connect(self.choose_directory)
+        self.directoryname = QtGui.QLineEdit(self)
+        self.directoryname.setMinimumWidth(300)
+        directorybox = QtGui.QHBoxLayout()
+        directorybox.addWidget(self.directorybutton)
+        directorybox.addWidget(self.directoryname)
+        return directorybox
+
+    def choose_file(self):
+        """
+        Opens a file dialog and sets the file text box to the chosen path.
+        """
+        dirname = self.get_default_directory(self.filename.text())
+        filename, _ = QtGui.QFileDialog.getOpenFileName(self, 'Open File',
+            dirname)
+        if os.path.exists(filename):    # avoids problems if <Cancel> was selected
+            dirname = os.path.dirname(filename)
+            self.filename.setText(str(filename))
+            self.set_default_directory(dirname)
+
+    def get_filename(self):
+        """
+        Returns the selected file.
+        """
+        return self.filename.text()
+
+    def choose_directory(self):
+        """
+        Opens a file dialog and sets the directory text box to the chosen path.
+        """
+        dirname = self.get_default_directory()
+        dirname = QtGui.QFileDialog.getExistingDirectory(self, 'Choose Directory',
+            dir=dirname)
+        if os.path.exists(dirname):    # avoids problems if <Cancel> was selected
+            self.directoryname.setText(str(dirname))
+            self.set_default_directory(dirname)
+
+    def get_directory(self):
+        """
+        Returns the selected directory
+        """
+        return self.directoryname.text()
+    
+    def get_default_directory(self, suggestion=None):
+        '''return the most recent default directory for open/save dialogs'''
+        if suggestion is None or not os.path.exists(suggestion):
+            suggestion = self.default_directory
+        if os.path.exists(suggestion):
+            if not os.path.isdir(suggestion):
+                suggestion = os.path.dirname(suggestion)
+        suggestion = os.path.abspath(suggestion)
+        return suggestion
+    
+    def set_default_directory(self, suggestion):
+        '''define the default directory to use for open/save dialogs'''
+        if os.path.exists(suggestion):
+            if not os.path.isdir(suggestion):
+                suggestion = os.path.dirname(suggestion)
+            self.default_directory = suggestion
+
+    def get_filesindirectory(self, prefix='', extension='.*'):
+        """
+        Returns a list of files in the selected directory.
+        
+        The files are sorted using a natural sort algorithm that preserves the
+        numeric order when a file name consists of text and index so that, e.g., 
+        'data2.tif' comes before 'data10.tif'.
+        """
+        os.chdir(self.get_directory())
+        if not extension.startswith('.'):
+            extension = '.'+extension
+        from glob import glob
+        filenames = glob(prefix+'*'+extension)
+        return sorted(filenames,key=natural_sort)
+
+    def read_parameter(self, root, path):
+        """
+        Read the value from the NeXus path.
+        
+        It will return 'None' if the path is not valid.
+        """
+        try:
+            value = root[path].nxdata
+            if isinstance(value, np.ndarray) and value.size == 1:
+                return np.float32(value)
+            else:
+                return value
+        except NeXusError:
+            return None 
+
+    def accept(self):
+        """
+        Accepts the result.
+        
+        This usually needs to be subclassed in each dialog.
+        """
+        self.accepted = True
+        QtGui.QDialog.accept(self)
+        
+    def reject(self):
+        """
+        Cancels the dialog without saving the result.
+        """
+        self.accepted = False
+        QtGui.QDialog.reject(self)
+
+    def update_progress(self):
+        """
+        Call the main QApplication.processEvents
+        
+        This ensures that GUI items like progress bars get updated
+        """
+        self.mainwindow._app.processEvents()
+
+    def progress_layout(self, save=False):
+        layout = QtGui.QHBoxLayout()
+        self.progress_bar = QtGui.QProgressBar()
+        layout.addWidget(self.progress_bar)
+        layout.addStretch()
+        layout.addWidget(self.buttonbox(save))
+        return layout
+
+    def get_node(self):
+        """
+        Return the node currently selected in the treeview
+        """
+        return self.treeview.get_node()
+
+    def confirm_action(self, query, information=None):
+        msgBox = QtGui.QMessageBox()
+        msgBox.setText(query)
+        if information:
+            msgBox.setInformativeText(information)
+        msgBox.setStandardButtons(QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel)
+        msgBox.setDefaultButton(QtGui.QMessageBox.Ok)
+        return msgBox.exec_()
+
+
+class PlotDialog(BaseDialog):
     """Dialog to plot arbitrary NeXus data in one or two dimensions"""
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(PlotDialog, self).__init__(parent)
  
         self.node = node
         self.dims = len(self.node.shape)
@@ -64,17 +261,10 @@ class PlotDialog(QtGui.QDialog):
         else:
             raise NeXusError('Item not plottable')
 
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         layout = QtGui.QVBoxLayout()
         for axis in range(self.dims):
             layout.addLayout(axis_layout[axis])
-        layout.addWidget(buttonbox) 
+        layout.addWidget(self.buttonbox()) 
         self.setLayout(layout)
 
         self.setWindowTitle("Plot NeXus Field")
@@ -128,13 +318,10 @@ class PlotDialog(QtGui.QDialog):
     def accept(self):
         data = NXdata(self.node, self.get_axes())
         data.plot()
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(PlotDialog, self).accept()
 
     
-class LimitDialog(QtGui.QDialog):
+class LimitDialog(BaseDialog):
     """Dialog to set plot window limits
     
     This is useful when it is desired to set the limits outside the data limits. 
@@ -142,7 +329,7 @@ class LimitDialog(QtGui.QDialog):
  
     def __init__(self, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(LimitDialog, self).__init__(parent)
  
         from nexpy.gui.plotview import plotview
 
@@ -177,14 +364,7 @@ class LimitDialog(QtGui.QDialog):
         ymax_layout.addWidget(self.ymax_box)
         layout.addLayout(ymax_layout)
 
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
-        layout.addWidget(buttonbox) 
+        layout.addWidget(self.buttonbox()) 
         self.setLayout(layout)
 
         self.setWindowTitle("Limit axes")
@@ -200,13 +380,10 @@ class LimitDialog(QtGui.QDialog):
         xmin, xmax = self.xmin_box.value(), self.xmax_box.value() 
         ymin, ymax = self.ymin_box.value(), self.ymax_box.value() 
         self.plotview.set_limits(xmin, xmax, ymin, ymax)
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(LimitDialog, self).accept()
 
     
-class AddDialog(QtGui.QDialog):
+class AddDialog(BaseDialog):
     """Dialog to add a NeXus node"""
 
     data_types = ['char', 'float32', 'float64', 'int8', 'uint8', 'int16', 
@@ -214,7 +391,7 @@ class AddDialog(QtGui.QDialog):
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(AddDialog, self).__init__(parent)
 
         self.node = node
 
@@ -237,16 +414,9 @@ class AddDialog(QtGui.QDialog):
         else:
             self.setWindowTitle("Add NeXus Data")
 
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         self.layout = QtGui.QVBoxLayout()
         self.layout.addLayout(class_layout)
-        self.layout.addWidget(buttonbox) 
+        self.layout.addWidget(self.buttonbox()) 
         self.setLayout(self.layout)
 
     def select_class(self):
@@ -401,21 +571,24 @@ class AddDialog(QtGui.QDialog):
                 self.node[name] = NXgroup(nxclass=nxclass)
             else:
                 self.node.insert(NXgroup(nxclass=nxclass))
+            logging.info("'%s' added to '%s'" 
+                         % (self.node[name], self.node.nxpath)) 
         elif name:
             value = self.get_value()
             dtype = self.get_type()
             if value is not None:
                 if self.class_name == "NXfield":
                     self.node[name] = NXfield(value, dtype=dtype)
+                    logging.info("'%s' added to '%s'" 
+                                 % (name, self.node.nxpath)) 
                 else:
                     self.node.attrs[name] = NXattr(value, dtype=dtype)
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+                    logging.info("Attribute '%s' added to '%s'" 
+                         % (name, self.node.nxpath)) 
+        super(AddDialog, self).accept()
 
     
-class InitializeDialog(QtGui.QDialog):
+class InitializeDialog(BaseDialog):
     """Dialog to initialize a NeXus field node"""
 
     data_types = ['char', 'float32', 'float64', 'int8', 'uint8', 'int16', 
@@ -423,7 +596,7 @@ class InitializeDialog(QtGui.QDialog):
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(InitializeDialog, self).__init__(parent)
  
         self.node = node
 
@@ -470,16 +643,9 @@ class InitializeDialog(QtGui.QDialog):
         grid.addWidget(self.shape_box, 3, 1)
         grid.setColumnMinimumWidth(1, 200)
 
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         self.layout = QtGui.QVBoxLayout()
         self.layout.addLayout(grid)
-        self.layout.addWidget(buttonbox) 
+        self.layout.addWidget(self.buttonbox()) 
         self.setLayout(self.layout)
 
     def select_combo(self):
@@ -518,33 +684,25 @@ class InitializeDialog(QtGui.QDialog):
                 dtype = np.float64
             shape = self.get_shape()
             self.node[name] = NXfield(dtype=dtype, shape=shape)
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+            logging.info("'%s' initialized in '%s'" 
+                         % (self.node[name], self.node.nxpath)) 
+        super(InitializeDialog, self).accept()
 
     
-class RenameDialog(QtGui.QDialog):
+class RenameDialog(BaseDialog):
     """Dialog to rename a NeXus node"""
 
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(RenameDialog, self).__init__(parent)
 
         self.node = node
 
         self.setWindowTitle("Rename NeXus data")
 
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         self.layout = QtGui.QVBoxLayout()
         self.layout.addLayout(self.define_grid())
-        self.layout.addWidget(buttonbox) 
+        self.layout.addWidget(self.buttonbox()) 
         self.setLayout(self.layout)
 
     def define_grid(self):
@@ -588,7 +746,7 @@ class RenameDialog(QtGui.QDialog):
             grid.addWidget(self.combo_box, 1, 1)
         else:
             parent_class = self.node.nxgroup.nxclass
-            if parent_class != 'NXroot' or parent_class != 'NXtree':
+            if parent_class != 'NXroot' and parent_class != 'NXtree':
                 combo_label = QtGui.QLabel()
                 combo_label.setAlignment(QtCore.Qt.AlignLeft)
                 combo_label.setText("Valid Fields:")
@@ -602,6 +760,10 @@ class RenameDialog(QtGui.QDialog):
                     self.combo_box.setItemData(self.combo_box.count()-1, 
                         wrap(_mainwindow.nxclasses[parent_class][1][name][2], 40),
                         QtCore.Qt.ToolTipRole)
+                if self.node.nxname in fields:
+                    self.combo_box.setCurrentIndex(self.combo_box.findText(self.node.nxname))
+                else:
+                    self.name_box.setText(self.node.nxname)
                 self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
                 grid.addWidget(self.combo_box, 0, 2)
         grid.setColumnMinimumWidth(1, 200)
@@ -623,82 +785,59 @@ class RenameDialog(QtGui.QDialog):
         if isinstance(self.node, NXgroup):
             if self.combo_box:
                 self.node.nxclass = self.get_class()
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(RenameDialog, self).accept()
 
     
-class RemoveDialog(QtGui.QDialog):
+class RemoveDialog(BaseDialog):
     """Dialog to remove a NeXus node from the tree"""
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(RemoveDialog, self).__init__(parent)
  
         self.node = node
  
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         layout = QtGui.QVBoxLayout()
         layout.addWidget(QtGui.QLabel('Are you sure you want to remove "%s"?' 
                                       % node.nxname))
-        layout.addWidget(buttonbox) 
+        layout.addWidget(self.buttonbox()) 
         self.setLayout(layout)
 
         self.setWindowTitle("Remove NeXus File")
 
     def accept(self):
         del self.node.nxgroup[self.node.nxname]
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(RemoveDialog, self).accept()
 
     
-class DeleteDialog(QtGui.QDialog):
+class DeleteDialog(BaseDialog):
     """Dialog to delete a NeXus node"""
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(DeleteDialog, self).__init__(parent)
  
         self.node = node
  
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         layout = QtGui.QVBoxLayout()
         layout.addWidget(QtGui.QLabel('Are you sure you want to delete "%s"?' 
                                       % node.nxname))
-        layout.addWidget(buttonbox) 
+        layout.addWidget(self.buttonbox()) 
         self.setLayout(layout)
 
         self.setWindowTitle("Delete NeXus Data")
 
     def accept(self):
         del self.node.nxgroup[self.node.nxname]
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(DeleteDialog, self).accept()
 
     
-class SignalDialog(QtGui.QDialog):
+class SignalDialog(BaseDialog):
     """Dialog to set the signal of NXdata"""
  
     def __init__(self, node, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(SignalDialog, self).__init__(parent)
  
         self.node = node
         self.dims = len(self.node.shape)
@@ -721,18 +860,11 @@ class SignalDialog(QtGui.QDialog):
             axis_layout[axis].addWidget(axis_label[axis])
             axis_layout[axis].addWidget(self.axis_boxes[axis])
  
-        buttonbox = QtGui.QDialogButtonBox(self)
-        buttonbox.setOrientation(QtCore.Qt.Horizontal)
-        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                     QtGui.QDialogButtonBox.Ok)
-        buttonbox.accepted.connect(self.accept)
-        buttonbox.rejected.connect(self.reject)
-
         layout = QtGui.QVBoxLayout()
         layout.addLayout(signal_layout)
         for axis in range(self.dims):
             layout.addLayout(axis_layout[axis])
-        layout.addWidget(buttonbox) 
+        layout.addWidget(self.buttonbox()) 
         self.setLayout(layout)
 
         self.setWindowTitle("Set %s as Signal" % self.node.nxname)
@@ -778,31 +910,24 @@ class SignalDialog(QtGui.QDialog):
         else:
             self.node.attrs["signal"] = signal
         self.node.axes = ":".join(self.get_axes())
-        QtGui.QDialog.accept(self)
-        
-    def reject(self):
-        QtGui.QDialog.reject(self)
+        super(SignalDialog, self).accept()
 
     
-class FitDialog(QtGui.QDialog):
+class FitDialog(BaseDialog):
     """Dialog to fit one-dimensional NeXus data"""
  
     def __init__(self, entry, parent=None):
 
-        QtGui.QDialog.__init__(self, parent)
+        super(FitDialog, self).__init__(parent)
+        self.setMinimumWidth(1000)        
  
-        self.data = entry.data
+        self.data = self.initialize_data(entry.data)
 
         from nexpy.gui.consoleapp import _tree
         self.tree = _tree
 
-        from nexpy.gui.plotview import change_plotview
-        self.plotview = change_plotview("Fit")
-        self.plotview.setMinimumSize(700, 300)
-        self.plotview.setMaximumSize(1200, 500)
-        
-        self.data.plot()
-        
+        from nexpy.gui.plotview import plotview
+        self.plotview = plotview
         self.functions = []
         self.parameters = []
 
@@ -827,7 +952,7 @@ class FitDialog(QtGui.QDialog):
         self.header_font = QtGui.QFont()
         self.header_font.setBold(True)
 
-        self.parameter_grid = self.initialize_parameter_grid()
+        self.parameter_layout = self.initialize_parameter_grid()
 
         self.remove_layout = QtGui.QHBoxLayout()
         remove_button = QtGui.QPushButton("Remove Function")
@@ -868,12 +993,13 @@ class FitDialog(QtGui.QDialog):
         fit_button = QtGui.QPushButton("Fit")
         fit_button.clicked.connect(self.fit_data)
         self.fit_label = QtGui.QLabel()
-        self.fit_checkbox = QtGui.QCheckBox('Use Errors')
         if self.data.nxerrors:
+            self.fit_checkbox = QtGui.QCheckBox('Use Errors')
             self.fit_checkbox.setCheckState(QtCore.Qt.Checked)
         else:
+            self.fit_checkbox = QtGui.QCheckBox('Use Poisson Errors')
             self.fit_checkbox.setCheckState(QtCore.Qt.Unchecked)
-            self.fit_checkbox.setVisible(False)
+            self.fit_checkbox.stateChanged.connect(self.define_errors)
         self.report_button = QtGui.QPushButton("Show Fit Report")
         self.report_button.clicked.connect(self.report_fit)
         self.save_button = QtGui.QPushButton("Save Parameters")
@@ -881,39 +1007,32 @@ class FitDialog(QtGui.QDialog):
         self.restore_button = QtGui.QPushButton("Restore Parameters")
         self.restore_button.clicked.connect(self.restore_parameters)
         self.action_layout.addWidget(fit_button)
-        self.action_layout.addWidget(self.fit_checkbox)
         self.action_layout.addWidget(self.fit_label)
         self.action_layout.addStretch()
+        self.action_layout.addWidget(self.fit_checkbox)
         self.action_layout.addWidget(self.save_button)
 
-        button_box = QtGui.QDialogButtonBox(self)
-        button_box.setOrientation(QtCore.Qt.Horizontal)
-        button_box.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
-                                      QtGui.QDialogButtonBox.Ok)
-        button_box.accepted.connect(self.accept)
-        button_box.rejected.connect(self.reject)
-
-        scroll_area = QtGui.QScrollArea()
-        scroll_area.setWidgetResizable(True)
-        self.scroll_widget = QtGui.QWidget()
-        self.scroll_widget.setMinimumWidth(800)
-        self.scroll_widget.setMaximumHeight(800)
-        self.scroll_widget.setSizePolicy(QtGui.QSizePolicy.Expanding, 
-                                         QtGui.QSizePolicy.Expanding)
         self.layout = QtGui.QVBoxLayout()
         self.layout.addLayout(function_layout)
-        self.layout.addWidget(button_box)
-        self.scroll_widget.setLayout(self.layout)
-        scroll_area.setWidget(self.scroll_widget)
-        layout = QtGui.QVBoxLayout()
-        layout.addWidget(self.plotview)
-        layout.addWidget(scroll_area)
-        self.setLayout(layout)
+        self.layout.addWidget(self.buttonbox())
+
+        self.setLayout(self.layout)
 
         self.setWindowTitle("Fit NeXus Data")
 
         self.load_entry(entry)
 
+    def initialize_data(self, data):
+        if isinstance(data, NXdata):
+            if len(data.nxsignal.shape) > 1:
+                raise NeXusError("Fitting only possible on one-dimensional arrays")
+            fit_data = NXdata(data.nxsignal, data.nxaxes, title=data.nxtitle)
+            if data.nxerrors:
+                fit_data.errors = data.nxerrors
+            return fit_data
+        else:
+            raise NeXusError("Must be an NXdata group")
+
     def initialize_functions(self):
 
         filenames = set()
@@ -943,8 +1062,13 @@ class FitDialog(QtGui.QDialog):
                 self.function_module[function_module.function_name] = function_module
 
     def initialize_parameter_grid(self):
-        grid = QtGui.QGridLayout()
-        grid.setSpacing(10)
+        grid_layout = QtGui.QVBoxLayout()
+        scroll_area = QtGui.QScrollArea()
+        scroll_area.setWidgetResizable(True)
+        scroll_widget = QtGui.QWidget()
+
+        self.parameter_grid = QtGui.QGridLayout()
+        self.parameter_grid.setSpacing(10)
         headers = ['Function', 'Np', 'Name', 'Value', '', 'Min', 'Max', 'Fixed', 'Bound']
         width = [100, 50, 100, 100, 100, 100, 100, 50, 100]
         column = 0
@@ -953,10 +1077,17 @@ class FitDialog(QtGui.QDialog):
             label.setFont(self.header_font)
             label.setAlignment(QtCore.Qt.AlignHCenter)
             label.setText(header)
-            grid.addWidget(label, 0, column)
-            grid.setColumnMinimumWidth(column, width[column])
+            self.parameter_grid.addWidget(label, 0, column)
+            self.parameter_grid.setColumnMinimumWidth(column, width[column])
             column += 1
-        return grid
+
+        scroll_widget.setLayout(self.parameter_grid)
+        scroll_area.setWidget(scroll_widget)
+        scroll_area.setMinimumHeight(200)
+        
+        grid_layout.addWidget(scroll_area)
+
+        return grid_layout
 
     def compressed_name(self, name):
         return re.sub(r'([a-zA-Z]*) # (\d*)', r'\1\2', name)
@@ -1000,7 +1131,6 @@ class FitDialog(QtGui.QDialog):
         self.guess_parameters()
         self.add_function_rows(f)
         self.write_parameters()
-        self.scroll_widget.adjustSize()
  
     def index_parameters(self):
         np = 0
@@ -1012,7 +1142,7 @@ class FitDialog(QtGui.QDialog):
     def add_function_rows(self, f):
         self.add_parameter_rows(f)
         if self.first_time:
-            self.layout.insertLayout(1, self.parameter_grid)
+            self.layout.insertLayout(1, self.parameter_layout)
             self.layout.insertLayout(2, self.remove_layout)
             self.layout.insertLayout(3, self.plot_layout)
             self.layout.insertLayout(4, self.action_layout)
@@ -1089,6 +1219,7 @@ class FitDialog(QtGui.QDialog):
             self.parameter_grid.addWidget(p.bound_box, row, 8)
             f.rows.append(row)
             row += 1
+        self.parameter_grid.setRowStretch(self.parameter_grid.rowCount(),10)
 
     def read_parameters(self):
         def make_float(value):
@@ -1151,6 +1282,10 @@ class FitDialog(QtGui.QDialog):
             f = filter(lambda x: x.name == name, self.functions)[0]
             self.get_model(f).oplot('--')
 
+    def define_errors(self):
+        if self.fit_checkbox.isChecked():
+            self.data.errors = np.sqrt(self.data.nxsignal)
+
     def fit_data(self):
         self.read_parameters()
         if self.fit_checkbox.isChecked():
@@ -1210,9 +1345,7 @@ class FitDialog(QtGui.QDialog):
         """Saves fit results to an NXentry"""
         self.read_parameters()
         entry = NXentry()
-        entry['title'] = 'Fit Results'
         entry['data'] = self.data
-        entry['fit'] = self.get_model()
         for f in self.functions:
             entry[f.name] = self.get_model(f)
             parameters = NXparameters()
@@ -1221,14 +1354,20 @@ class FitDialog(QtGui.QDialog):
                                              initial_value=p.init_value,
                                              min=str(p.min), max=str(p.max))
             entry[f.name].insert(parameters)
-        fit = NXparameters()
-        fit.nfev = self.fit.result.nfev
-        fit.ier = self.fit.result.ier 
-        fit.chisq = self.fit.result.chisqr
-        fit.redchi = self.fit.result.redchi
-        fit.message = self.fit.result.message
-        fit.lmdif_message = self.fit.result.lmdif_message
-        entry['statistics'] = fit
+        if self.fit is not None:
+            entry['title'] = 'Fit Results'
+            entry['fit'] = self.get_model()
+            fit = NXparameters()
+            fit.nfev = self.fit.result.nfev
+            fit.ier = self.fit.result.ier 
+            fit.chisq = self.fit.result.chisqr
+            fit.redchi = self.fit.result.redchi
+            fit.message = self.fit.result.message
+            fit.lmdif_message = self.fit.result.lmdif_message
+            entry['statistics'] = fit
+        else:
+            entry['title'] = 'Fit Model'
+            entry['model'] = self.get_model()
         if 'w0' not in self.tree.keys():
             self.tree.add(NXroot(name='w0'))
         ind = []
@@ -1252,13 +1391,11 @@ class FitDialog(QtGui.QDialog):
         self.write_parameters()
    
     def accept(self):
-        self.plotview.close_view()
-        QtGui.QDialog.accept(self)
+        super(FitDialog, self).accept()
         
     def reject(self):
-        self.plotview.close_view()
-        QtGui.QDialog.reject(self)
+        super(FitDialog, self).reject()
 
     def closeEvent(self, event):
-        self.plotview.close_view()
         event.accept()
+
