@@ -45,7 +45,7 @@ def new_figure_manager(label=None, *args, **kwargs):
     """
     if label is None:
         label = ''
-    if 'Projection' in label or 'Fit' in label:
+    if 'Projection' in label:
         nums = [num for num in plt.get_fignums() if num > 100]
         if nums:
             num = max(nums) + 1
@@ -156,9 +156,12 @@ class NXPlotView(QtGui.QWidget):
 
         Gcf.set_active(self.figuremanager)
         def make_active(event):
-            if 'Projection' not in self.label and 'Fit' not in self.label:
+            if 'Projection' not in self.label:
                 self.make_active()
-            if event.button == 3:
+            if event.button == 1:
+                self.xdata = event.xdata
+                self.ydata = event.ydata
+            elif event.button == 3:
                 hasattr(self, 'otab')
                 self.otab.home()
         cid = self.canvas.mpl_connect('button_press_event', make_active)
@@ -202,7 +205,7 @@ class NXPlotView(QtGui.QWidget):
         plotviews[self.label] = plotview
         self.plotviews = plotviews
 
-        if self.label not in ['Main', 'Fit'] and 'Projection' not in self.label:
+        if self.label != "Main":
             self.add_menu_action()
 
         self.show()
@@ -227,7 +230,7 @@ class NXPlotView(QtGui.QWidget):
         self.update_active()
 
     def update_active(self):
-        if 'Projection' not in self.label and 'Fit' not in self.label:
+        if 'Projection' not in self.label:
             from nexpy.gui.consoleapp import _mainwindow
             _mainwindow.update_active(self.label)
     
@@ -1382,7 +1385,8 @@ class NXProjectionTab(QtGui.QWidget):
         
         self.overplot_box = QtGui.QCheckBox("Over")
         self.overplot_box.setChecked(False)
-        self.overplot_box.setVisible(False)
+        if 'Projection' not in plotviews:
+            self.overplot_box.setVisible(False)
         widgets.append(self.overplot_box)
 
         self.panel_button = QtGui.QPushButton("Open Panel", self)
@@ -1478,7 +1482,7 @@ class NXProjectionTab(QtGui.QWidget):
 
     def plot_projection(self):
         axes, limits = self.get_projection()
-        projection = change_plotview("Projection - " + self.plotview.label)
+        projection = change_plotview("Projection")
         if len(axes) == 1 and self.overplot_box.isChecked():
             over = True
         else:
@@ -1585,7 +1589,8 @@ class NXProjectionPanel(QtGui.QDialog):
         grid.addWidget(self.plot_button, row, 2)
         self.overplot_box = QtGui.QCheckBox()
         self.overplot_box.setChecked(False)
-        self.overplot_box.setVisible(False)
+        if 'Projection' not in plotviews:
+            self.overplot_box.setVisible(False)
         grid.addWidget(self.overplot_box, row, 3, 
                        alignment=QtCore.Qt.AlignHCenter)
 
@@ -1743,7 +1748,7 @@ class NXProjectionPanel(QtGui.QDialog):
     def plot_projection(self):
         try:
             axes, limits = self.get_projection()
-            projection = change_plotview("Projection - " + self.plotview.label)
+            projection = change_plotview("Projection")
             if len(axes) == 1 and self.overplot_box.isChecked():
                 over = True
             else:
@@ -1831,14 +1836,29 @@ class NXNavigationToolbar(NavigationToolbar):
         self.zoom()
 
     def _init_toolbar(self):
-
+        if not hasattr(self, 'toolitems'):
+            self.toolitems = (
+                ('Home', 'Reset original view', 'home', 'home'),
+                ('Back', 'Back to  previous view', 'back', 'back'),
+                ('Forward', 'Forward to next view', 'forward', 'forward'),
+                (None, None, None, None),
+                ('Pan', 'Pan axes with left mouse, zoom with right', 'move', 'pan'),
+                ('Zoom', 'Zoom to rectangle', 'zoom_to_rect', 'zoom'),
+                (None, None, None, None),
+                ('Subplots', 'Configure subplots', 'subplots', 'configure_subplots'),
+                ('Save', 'Save the figure', 'filesave', 'save_figure'),
+                )
         self.toolitems = list(self.toolitems)
         self.toolitems.append(('Add', 'Add plot data to the tree', 'hand', 'add_data'))
         super(NXNavigationToolbar, self)._init_toolbar()
 
     def home(self, *args):
-        self.plotview.reset_limits()
         super(NXNavigationToolbar, self).home()        
+        self.plotview.reset_limits()
+        xmin, xmax = self.plotview.plot.xaxis.min, plotview.plot.xaxis.max
+        plotview.xtab.set_limits(xmin, xmax)
+        ymin, ymax = self.plotview.plot.yaxis.min, plotview.plot.yaxis.max
+        plotview.ytab.set_limits(ymin, ymax)
 
     def add_data(self):
         keep_data(self.plotview.plot.plotdata)
